/**
 * @fileoverview Sort tailwind class names
 * @author Kasper Mikiewicz
 */
'use strict'

const defaultSortOrder = require('../helpers/default-sort-order')

//------------------------------------------------------------------------------
// Rule Definition
//------------------------------------------------------------------------------

module.exports = {
  meta: {
    docs: {
      description: 'Sort tailwind class names',
      category: 'Stylistic Issues',
      recommended: true,
    },
    fixable: 'code', // or "code" or "whitespace"
    schema: [
      // fill in your schema
    ],
  },

  create: function(context) {
    const sortClassString = (
      classString,
      sortOrder,
      shouldRemoveDuplicates
    ) => {
      let classArray = classString.split(/\s+/g)

      if (shouldRemoveDuplicates) {
        classArray = removeDuplicates(classArray)
      }

      classArray = sortClassArray(classArray, sortOrder)

      return classArray.join(' ')
    }

    const sortClassArray = (classArray, sortOrder) => [
      ...classArray
        .filter(el => sortOrder.indexOf(el) !== -1) // take the classes that are in the sort order
        .sort((a, b) => sortOrder.indexOf(a) - sortOrder.indexOf(b)), // and sort them
      ...classArray.filter(el => sortOrder.indexOf(el) === -1), // prepend the classes that were not in the sort order
    ]

    const removeDuplicates = classArray => [...new Set(classArray)]

    //----------------------------------------------------------------------
    // Public
    //----------------------------------------------------------------------

    return {
      JSXAttribute(node) {
        if (node.name.name !== 'class') return
        if (!node.value) return
        if (node.value.type !== 'Literal') return
        const classNames = node.value.value
        const sortedClases = sortClassString(classNames, defaultSortOrder, true)
        if (sortedClases === classNames) return

        context.report({
          node,
          loc: {
            start: node.value.loc.start,
            end: node.value.loc.end,
          },
          message: 'Invalid class names order',
          fix(fixer) {
            return [
              fixer.replaceTextRange(
                [node.value.start + 1, node.value.end - 1],
                sortedClases
              ),
            ]
          },
        })
      },
    }
  },
}
